import { useUserStore } from '@/stores';
import { type PageErrorReason, usePageStore } from '@/stores/page';
import { IdPool, waitRef } from '@/utils';
import { useThrottleFn } from '@vueuse/core';
import { toRef } from 'vue';
import { createRouter, createWebHistory, type RouteRecordRaw } from 'vue-router';
import { RoutePermissionId } from './permissions';

export * from './permissions';

declare module 'vue-router' {
  interface RouteMeta {
    errorRouteName?: string;
    routeId?: number;
    permissionId?: RoutePermissionId;
    pageErrorReason?: PageErrorReason;
  }
}
const routes: RouteRecordRaw[] = [
  {
    path: '/',
    component: () => import('@/views/MainPage.vue')
  },
  {
    path: '/user',
    component: () => import('@/views/UserPage.vue'),
    meta: {
      permissionId: RoutePermissionId.USER_PAGE
    }
  },
  {
    path: '/user/:id',
    component: () => import('@/views/UserPage.vue')
  },
  {
    path: '/club',
    component: () => import('@/views/ClubPage.vue'),
    meta: {
      permissionId: RoutePermissionId.CLUB_PAGE
    }
  },
  {
    path: '/2048',
    component: () => import('@/views/Game2048Page.vue')
  },
  {
    path: '/:path(.*)*',
    name: 'NotFound',
    component: () => import('@/views/ErrorPage.vue')
  }
];
(function processRoutes(routes: RouteRecordRaw[], idPool: IdPool) {
  for (const route of routes) {
    if (route.name === undefined) {
      route.name = `AnonymousPage${idPool.newId()}`;
      processRoutes(route.children ?? [], idPool);
    }
  }
})(routes, new IdPool(1));
export const tempErrorPageName = '_TempError';

const router = createRouter({
  history: createWebHistory(),
  routes: routes
});

router.beforeEach(async (to, from) => {
  const userStore = useUserStore();
  const pageStore = usePageStore();
  const { permissionId } = to.meta;
  pageStore.setNewRouteId(to);
  try {
    switch (userStore.userInfoStatus) {
      case 'uninitialized': {
        const succeed = await userStore.updateSelfUserInfo(true);
        if (!succeed) {
          if (permissionId === undefined) {
            return;
          }
          return pageStore.createTempErrorRoute(
            {
              type: 'networkError',
              originalPath: to.fullPath
            },
            to
          );
        }
        break;
      }
      case 'initializing':
        return false;
    }
    if (permissionId !== undefined) {
      if (userStore.hasPermission(permissionId)) {
        return true;
      } else {
        return pageStore.createTempErrorRoute({ type: 'noPermission' }, from);
      }
    }
  } finally {
    pageStore.removeRouteId(to);
  }
});
router.beforeEach((to) => {
  const pageStore = usePageStore();
  switch (to.name) {
    case 'NotFound':
      pageStore.pageErrorReason = { type: 'notFound' };
      break;
    case undefined:
      pageStore.pageErrorReason = undefined;
  }
});
// router.afterEach(
//   useThrottleFn(
//     async (to) => {
//       const userStore = useUserStore();
//       const pageStore = usePageStore();
//       if (['initialized', 'failed'].includes(userStore.userInfoStatus)) {
//         await waitRef(toRef(userStore, 'userInfoStatus'), 'initialized');
//       }
//       await userStore.updateSelfUserInfo(false, true);
//       const { permissionId } = to.meta;
//       if (permissionId !== undefined && !userStore.hasPermission(permissionId)) {
//         router.push(pageStore.createTempErrorRoute({ type: 'noPermission' }));
//       }
//     },
//     10000,
//     true
//   )
// );
router.afterEach((to) => {
  const pageStore = usePageStore();
  pageStore.removeRouteId(to);
  if (to.name === tempErrorPageName) {
    router.removeRoute(tempErrorPageName);
  }
});
export default router;
