import axiosInstance, { type RawResp } from '@/api';
import type { UserInfo } from '@/schemas';
import { userInfoResponseSchema, type idAndNameSchema } from '@/schemas';
import { errorMessage } from '@/utils';
import { StorageSerializers, useLocalStorage } from '@vueuse/core';
import { AxiosError } from 'axios';
import { ElMessage } from 'element-plus';
import { defineStore } from 'pinia';
import { computed, ref, watch } from 'vue';
import { z } from 'zod';

type Permission = z.infer<typeof idAndNameSchema>;
export const useUserStore = defineStore('user', () => {
  const token = useLocalStorage<string | null>('token', null, {
    serializer: StorageSerializers.string
  });
  const userInfo = useLocalStorage<UserInfo | null>('user-info', null, {
    serializer: StorageSerializers.object
  });
  const permissions = computed<Permission[]>(() => userInfo.value?.auth.permissions ?? []);
  const userInfoStatus = ref<'uninitialized' | 'initializing' | 'initialized'>('uninitialized');
  watch(
    userInfo,
    (info) => {
      userInfoStatus.value = info ? 'initialized' : 'uninitialized';
    },
    { flush: 'sync' }
  );
  async function updateSelfUserInfo(
    showErrorMessage: boolean,
    silent: boolean = false
  ): Promise<boolean> {
    if (!silent) userInfoStatus.value = 'initializing';
    const raw = await axiosInstance
      .get<RawResp>('/api/user/info')
      .then((r) => r.data)
      .catch((e: AxiosError) => {
        if (!showErrorMessage) return;
        ElMessage.error(errorMessage('获取用户信息失败', e.code, e.message));
      });
    const resp = userInfoResponseSchema.parse(raw);
    if (resp.type === 'error') {
      if (showErrorMessage) {
        ElMessage.error(errorMessage('获取用户信息失败', resp.code, resp.msg));
      }
      return false;
    }
    userInfo.value = resp.data;
    return true;
  }
  function hasPermission(permissionId: number) {
    return permissions.value.find((permission) => permission.id === permissionId);
  }

  function $reset() {
    token.value = null;
    userInfo.value = null;
    userInfoStatus.value = 'uninitialized';
  }

  return {
    token,
    userInfo,
    permissions,
    userInfoStatus,
    updateSelfUserInfo,
    hasPermission,
    $reset
  };
});
